package fxKaljoja;

import javafx.scene.control.Alert;
import javafx.scene.control.ButtonBar;
import javafx.scene.control.ButtonType;
import javafx.scene.control.ComboBox;
import javafx.scene.input.KeyEvent;

import java.util.*;

/**
 * Oma versio ComboBox containerista, sisältää haun ja uuden lisäyksen tulkinnan
 */
public class MynamicComboBox<T> extends ComboBox<String> {

    /**
     * Attribuutti items sisältää kaiken comboxin sisällön
     */
    private List<String> items = new ArrayList<>();

    public MynamicComboBox() {
        super();
        this.getEditor().addEventHandler(KeyEvent.KEY_TYPED, this::haku);
        this.getEditor().setOnAction(_ -> tutkiValinta());
    }
    /**
     * Seuraavat metodit ovat molemmat sisallon lisäämistä/asettamsita varten
     *
     * @param sisalto mikä merkkijono/sisalto kokoelma comboboxiin laitetaan
     */
    public void asetaSisalto(String[] sisalto) {
        items.addAll(Arrays.asList(sisalto));
        for (String s : sisalto) {
            this.getItems().add(s);
        }
    }

    public void asetaSisalto(List<String> sisalto) {
        items.addAll(sisalto);
        for (String s : sisalto) {
            this.getItems().add(s);
        }

    }


    /**
     * lisää sisältöä comboboxiin
     *
     * @param sisalto merkkijono, joka lisätään
     */
    public void lisaaSisaltoa(String sisalto) {
        items.add(sisalto);
        this.getItems().add(sisalto);
    }


    /**
     * Vallinan tutkimis metodi Tutkii mitä käyttäjä on sijoittanut ComboBoxiin, jos on uusi valinta,
     * niin kysytään halutaanko se lisätä valintoihin. Instanssi pohjainen lisäys, jos lisäys halutaan
     * lopulliseksi tulee se tehdä itse
     */
    private void tutkiValinta() {
        String valinta = this.getEditor().getText();
        if (valinta == null) {
            return;
        }
        valinta = valinta.trim();
        if (valinta.isEmpty()) {
            return;
        }
        // katsotaan löytyykö valinta
        boolean loytyko = false;
        for (String item : items) {
            if (valinta.equals(item)) {
                loytyko = true;
                break;
            }

        }
        //jos ei löytynyt, niin avataan dialogi ja kysytään halutaanko valinta lisätä
        if (!loytyko) {
            tallennusIkkuna(valinta);
        } else {
            this.getEditor().requestFocus();
        }
    }


    /**
     * Metodi comboboxista etsimistä varten, Algoritmi toimii vertaamalla käyttäjän tekstiä
     * items listan sisältöihin ja lisää ne comboxiin, mitkä vastaavat hakusanaa.
     * Jos haku tyhjä, niin laitetaan kaikki comboboxiin, tulostetaan ja poistutaan.
     * Aikavaativuus kasvaa lineaarisesti, items listan kasvaessa
     */
    private void haku(KeyEvent k) {
        //estetään hakeminen, jos painettu enter-nappia
        if (k.getCharacter().equals("\r")) {
            tutkiValinta();
            return;
        }
        String hakuSana = this.getEditor().getText() + k.getCharacter();
        hakuSana = hakuSana.trim();
        //varmistetaan, että hakusana ei ole tyhjä, jos on niin näytetään kaikki vaihtoehdot
        if (hakuSana.isEmpty()) {
            this.getItems().setAll(items);
            this.show();
            return;
        }
        System.out.println("hakuSana: " + hakuSana); //logging
        //käydään läpi jokainen vaihtoehto ja näytetään vain ne jotka ovat yhteensopivia
        List<String> naytettavat = new ArrayList<>();
        for (String item : items) {
            if (item.startsWith(hakuSana)) {
                naytettavat.add(item);
            }
        }
        this.getItems().setAll(naytettavat);
    }


    /**
     * Tallennusikkunan esittävä metodi
     *
     * @param valinta valinta joka tallennetaan
     */
    private void tallennusIkkuna(String valinta) {
        Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
        alert.setTitle("Valitse");
        alert.setHeaderText(null);
        alert.setContentText("Haluatko tallentaa valinnan " + valinta + "?");

        ButtonType buttonTypeYes = new ButtonType("Kyllä", ButtonBar.ButtonData.OK_DONE);
        ButtonType buttonTypeCancel = new ButtonType("Ei", ButtonBar.ButtonData.CANCEL_CLOSE);

        alert.getButtonTypes().setAll(buttonTypeYes, buttonTypeCancel);

        Optional<ButtonType> result = alert.showAndWait();
        if (result.get() == buttonTypeYes) {
            items.add(valinta);
            this.getItems().add(valinta);
            System.out.println("Talletaan");
        }
    }

}


